import { world, system } from "@minecraft/server";

const TARGET_ENTITY_TYPE = "pa:twisted_rudie";

world.afterEvents.entitySpawn.subscribe((data) => {
    const entity = data.entity;

    if (entity.typeId === TARGET_ENTITY_TYPE) {
        let lastPosition = null;

        const followLight = system.runInterval(() => {
            if (!entity.isValid()) {
                if (lastPosition) {
                    const { x, y, z } = lastPosition;
                    entity.dimension.runCommandAsync(`setblock ${x} ${y} ${z} air`);
                }
                system.clearRun(followLight);
                return;
            }

            const { x, y, z } = entity.location;
            const dimension = entity.dimension;

            const currentPosition = {
                x: Math.floor(x),
                y: Math.floor(y) + 1,
                z: Math.floor(z),
            };

            if (
                !lastPosition ||
                currentPosition.x !== lastPosition.x ||
                currentPosition.y !== lastPosition.y ||
                currentPosition.z !== lastPosition.z
            ) {
                if (lastPosition) {
                    const { x: lastX, y: lastY, z: lastZ } = lastPosition;
                    dimension.runCommandAsync(`setblock ${lastX} ${lastY} ${lastZ} air`);
                }

                dimension.runCommandAsync(`setblock ${currentPosition.x} ${currentPosition.y} ${currentPosition.z} light_block_7`);
                lastPosition = currentPosition;
            }
        }, 1);

        world.afterEvents.entityDie.subscribe((event) => {

            const entity = event.deadEntity;

            if (entity.typeId === TARGET_ENTITY_TYPE) {
                if (lastPosition) {
                    const { x, y, z } = lastPosition;
                    entity.dimension.runCommandAsync(`setblock ${x} ${y} ${z} air`);
                }
                system.clearRun(followLight);
            }
        });
    }
});